/* global nsVmap, ol, oVmap, goog, angular */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.Map
 * cette classe permet l'initialisation de la carte
 */

goog.provide('nsVmap.Map.MapPopup');
goog.require('ol.Overlay');

/**
 * @classdesc
 * Class {@link nsVmap.Map.MapPopup}: A tooltip on the map
 *
 * @param {ol.Feature} olFeature Feature to bind with the popup
 * @param {ol.geom.Point|undefined} olPoint point where to put the popup
 * @constructor
 * @export
 */
nsVmap.Map.MapPopup = function (olFeature, olPoint, opt_options) {
    oVmap.log('nsVmap.Map.MapPopup');

    opt_options = goog.isDefAndNotNull(opt_options) ? opt_options : {};

    /**
     * @type nsVmap.Map.MapPopup
     * @private
     */
    var this_ = this;

    /**
     * @type ol.Feature
     * @private
     */
    this.olFeature_ = olFeature;

    /**
     * @type ol.Map
     * @private
     */
    this.map_ = oVmap.getMap().getOLMap();

    /**
     * @type ol.proj
     * @private
     */
    this.projection_ = this.map_.getView().getProjection().getCode();

    /**
     * Actions to add to the popup
     */
    this.actions = [];

    /**
     * Style à utiliser
     */
    this.oStyle = {};

    /**
     * Popup active ou inactive
     */
    this.active = opt_options.active === false ? false : true;

    /**
     * Liste des autres popups
     */
    this.aPopupsList;

    /**
     * liste des boutons du menu popup list
     */
    this.aPopupListMenuBtns;

    /**
     * Donée table
     */
    this.tableData;

    /**
     * Module de traduction
     */
    this.translate_ = angular.element(vitisApp.appMainDrtv).injector().get(['$translate']);

    // Ajoute la feature sur la carte
    if (!goog.array.contains(oVmap.getMap().getPopupOverlaySource().getFeatures(), olFeature))
        oVmap.getMap().getPopupOverlaySource().addFeature(olFeature);

    // Création du HTML relatif
    this.HTML_ = document.createElement('div');
    this.HTML_.className = 'ol-popup';

    this.HTMLContainer_ = document.createElement('div');
    if (oVmap['properties']['is_mobile']) {
      this.HTMLContainer_.className = 'mobile-popup';
    }

    this.HTML_.appendChild(this.HTMLContainer_);

    this.HTMLButtonsContainer_ = document.createElement('div');
    this.HTMLButtonsContainer_.className = 'ol-popup-buttons-container';
    this.HTMLContainer_.appendChild(this.HTMLButtonsContainer_);

    // Ajoute le style défini en properties
    if (goog.isDefAndNotNull(oVmap['properties']['popup'])) {
        if (goog.isDefAndNotNull(oVmap['properties']['popup']['style'])) {
            if (goog.isString(oVmap['properties']['popup']['style'])) {

                // Style définit en properties en enlevant les "\n"
                var sStyle = oVmap['properties']['popup']['style'].replace(/<return>/g, '');
                var aStyle1 = sStyle.split(';');

                for (var i = 0; i < aStyle1.length; i++) {
                    var aStyle2 = aStyle1[i].split(':');
                    if (goog.isDefAndNotNull(aStyle2[0]) && goog.isDefAndNotNull(aStyle2[1])) {
                        this.oStyle[aStyle2[0]] = aStyle2[1];
                    }
                    delete aStyle2;
                }
                for (var key in this.oStyle) {
                    this.HTML_.style[key] = this.oStyle[key];
                }
            }
        }
    }

    // Bouton close
    this.HTMLCloser_ = document.createElement('a');
    this.HTMLCloser_.href = '#';
    this.HTMLCloser_.className = 'ol-popup-closer';
    this.HTMLButtonsContainer_.appendChild(this.HTMLCloser_);

    this.HTMLContent_ = document.createElement('div');
    this.HTMLContent_.classList.add('popup-content');
    this.HTMLContainer_.appendChild(this.HTMLContent_);


    /**
     * Add a click handler to hide the popup.
     * @return {boolean} Don't follow the href.
     */
    this.HTMLCloser_.onclick = function () {
        this_.remove();
        return false;
    };

    // Zoom la carte quand l'utilisateur utilisa la molette sur la popup
    var lastWheelEvent = Date.now();
    this.HTML_.addEventListener("wheel", function (event) {
        if (Date.now() - lastWheelEvent > 200) {
            if (event.deltaY > 0) {
                this_.map_.getView().setZoom(this_.map_.getView().getZoom() - 1);
            }
            if (event.deltaY < 0) {
                this_.map_.getView().setZoom(this_.map_.getView().getZoom() + 1);
            }
            lastWheelEvent = Date.now();
        }
    });

    if (oVmap['properties']['is_mobile']) {
        $('#mobile-popup-bar').removeClass('hidden');
        $('#mobile-popup-bar').append(this.HTML_.children);
        this.HTML_ = $('#mobile-popup-bar').children('.mobile-popup').last()[0];

        // Set active popup
        this.setActive(this.active);
    } else {

        /**
         * @type {ol.Overlay}
         * @private
         */
        this.popupOverlay_ = new ol.Overlay(({
            element: this.HTML_,
            autoPan: true,
            position: goog.isDefAndNotNull(olPoint) ? this.getGeometryCenter(olPoint) : this.getGeometryCenter(olFeature.getGeometry()),
            autoPanAnimation: {
                duration: 250
            }
        }));

        this.map_.addOverlay(this.popupOverlay_);

        // Change de coordonnées lorsqu'on change de projection
        this.map_.on('change:view', function () {

            var oldProj = this.projection_;
            var oldCoord = this.popupOverlay_.getPosition();

            if (!goog.isDef(oldCoord))
                return 0;
            if (!goog.isDef(oldProj))
                return 0;

            var newProj = this.map_.getView().getProjection().getCode();
            var newCoord = ol.proj.transform(oldCoord, oldProj, newProj);

            this.popupOverlay_.setPosition(newCoord);
            this.projection_ = newProj;

        }, this);

        // Définit les actions quand on clique sur la popup
        this.handlePopupClick();
        
        // Set active popup
        this.setActive(this.active);
    }
};

/**
 * Display a message
 * @param {string} message
 * @returns {undefined}
 * @export
 */
nsVmap.Map.MapPopup.prototype.displayMessage = function (message) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.displayMessage');
    this.removeMessages();
    this.addMessage(message);
};

/**
 * Display a table on the popup according to the object
 * @param {object} data data to display
 * @returns {Boolean}
 */
nsVmap.Map.MapPopup.prototype.displayMessageTable = function (data) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.displayMessage');
    this.removeMessages();
    this.addMessageTable(data);
};

/**
 * Add a message
 * @param {string} message
 * @returns {undefined}
 * @export
 */
nsVmap.Map.MapPopup.prototype.addMessage = function (message) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.displayMessage');

    if (!goog.isDef(message)) {
        console.error('MapPopup: message not defined');
        return 0;
    }

    this.HTMLContent_.innerHTML = message;
    return true;
};

/**
 * 
 * @param {string} photoURL
 * @returns {Number|Boolean}
 * @export
 */
nsVmap.Map.MapPopup.prototype.addPhoto = function (photoURL) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.addPhoto');

    if (!goog.isDef(photoURL)) {
        console.error('MapPopup: addPhoto not defined');
        return 0;
    }

    var this_ = this;

    var img = document.createElement("img");
    img.style.width = '100px';
    img.src = photoURL;

    if (!goog.isDefAndNotNull(this.oStyle['min-width']) || $(this_.HTML_).css('min-width') === '0px') {
        $(this_.HTML_).css('min-width', '300px');
    }

    if (!goog.isDef(this.HTMLImageContainer_)) {

        this.HTMLImageContainer_ = document.createElement("div");
        this.HTMLImageContainer_.classList.add('popup-image-content');
        this.HTMLImageContainer_.style['position'] = 'relative';
        this.HTMLImageContainer_.style['float'] = 'right';
        this.HTMLContent_.style['float'] = 'left';
        this.HTMLContent_.style['width'] = 'calc(100% - 100px)';
        this.HTMLContainer_.appendChild(this.HTMLImageContainer_);
    }

    this.HTMLImageContainer_.appendChild(img);

    return true;
};

/**
 * Add a table on the popup according to the object
 * @param {object} data data to display
 * @returns {Boolean}
 */
nsVmap.Map.MapPopup.prototype.addMessageTable = function (data) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.addMessageTable');

    this.tableData = data;

    if (!goog.isDef(data)) {
        console.error('MapPopup: data not defined');
        return 0;
    }

    var HTMLMessageContainer = document.createElement('table');

    var aAllowedFunctions = ['getPropertie'];

    for (var item in data) {

        // car feature est le mot clé utilisé pour stocker la feature dans infocontainer
        if (item === 'feature')
            continue;

        var row = document.createElement('tr');

        var column1 = document.createElement('td');
        column1.className = 'padding-sides-5 map-popup-cell';
        row.appendChild(column1);

        var column2 = document.createElement('td');
        column2.className = 'padding-sides-5 map-popup-cell';
        row.appendChild(column2);

        if (goog.isString(data[item])) {
            
            // Recherche les liens
            if (oVmap.isLink(data[item], 'bo_link')) {
                data[item] = oVmap.parseLink(data[item], 'bo_link');
            }
            
            // Prévention d'injections XSS
            data[item] = oVmap['$sanitize'](data[item]);
            
            // Recherche les fonctions autorisées
            for (var i = 0; i < aAllowedFunctions.length; i++) {
                if (data[item].indexOf('{{' + aAllowedFunctions[i] + '(') !== -1 && data[item].indexOf(')}}') !== -1) {
                    data[item] = this.parseFunction_(data[item], aAllowedFunctions[i]);
                }
            }

            // Recherche les liens
            if (oVmap.isDate(data[item])) {
                data[item] = oVmap.formatDate(data[item]);
            }
        }

        column1.innerHTML = item;
        column2.innerHTML = data[item];

        HTMLMessageContainer.appendChild(row);

        delete row;
        delete column1;
        delete column2;
    }

    this.HTMLContent_.appendChild(HTMLMessageContainer);
    return true;
};

/**
 * Parse and execute a function defines in sString
 * @param {string} sString
 * @returns {string}
 */
nsVmap.Map.MapPopup.prototype.parseFunction_ = function (sString) {

    var sFunction = sString.substr(sString.indexOf('{{') + 2, sString.indexOf('}}') - sString.indexOf('{{') - 2);
    var sResult = '';

    if (sFunction.split(';').length > 1) {
        console.error('Attention: plusieurs fonctions détectées dans ' + sString);
    }

    try {
        // Split vérifie qu'il n'y a qu'une seule fonction à executer
        sResult = eval('angular.element(vitisApp.appMainDrtv).scope().' + sFunction.split(';')[0]);
    } catch (e) {

    }

    sString = sString.replace('{{' + sFunction + '}}', sResult);

    return sString;
};

/**
 * Add an action button on the popup
 * @param {object} action
 * @param {object} action.content content of the buton
 * @param {object} action.event event onclick
 * @export
 */
nsVmap.Map.MapPopup.prototype.addAction = function (action) {

    if (this.actions.indexOf(action) === -1) {
        this.actions.push(action);
        this.compileActions();
    }
};

/**
 * Remove an action button from the popup
 * @param {object} action action to remove
 * @export
 */
nsVmap.Map.MapPopup.prototype.removeAction = function (action) {
    if (this.actions.indexOf(action) !== -1) {
        this.actions.splice(this.actions.indexOf(action), 1);
        this.compileActions();
    }
};

/**
 * Compile the actions by this.actions
 * @export
 */
nsVmap.Map.MapPopup.prototype.compileActions = function () {
    oVmap.log('nsVmap.Map.MapPopup.prototype.compileActions');

    var this_ = this;

    // Container
    if (goog.isDef(this.HTMLDropup_))
        this.HTMLButtonsContainer_.removeChild(this.HTMLDropup_);
    this.HTMLDropup_ = document.createElement('div');
    this.HTMLDropup_.className = 'dropup';
    this.HTMLDropup_.style.position = 'inherit';

    // Bouton dropup
    this.HTMLList_ = document.createElement('a');
    this.HTMLList_.href = '#';
    this.HTMLList_.className = 'ol-popup-list icon-format_list_bulleted';
    this.HTMLList_.addEventListener('click', function () {
        $(this_.HTMLListUl_).toggleClass('block');
    });
    $('body').click(function (e) {
        if ($(this_.HTMLList_)[0] !== $(e.target)[0]){
            $(this_.HTMLListUl_).removeClass('block');
        }
    });

    // Liste
    this.HTMLListUl_ = document.createElement('ul');
    this.HTMLListUl_.style = 'right: 0;left: inherit;';
    this.HTMLListUl_.className = 'dropdown-menu';

    for (var i = 0; i < this.actions.length; i++) {
        this.HTMLListLi_ = document.createElement('li');
        this.HTMLListLiA_ = document.createElement('a');
        this.HTMLListLiA_.href = '#';
        this.HTMLListLiA_.innerHTML = this.actions[i]['content'];
        this.HTMLListLiA_.addEventListener('click', this.actions[i]['event']);
        this.HTMLListLiA_.addEventListener('click', function () {
            $(this_.HTMLListUl_).removeClass('block');
        });

        this.HTMLListUl_.appendChild(this.HTMLListLi_);
        this.HTMLListLi_.appendChild(this.HTMLListLiA_);
    }

    this.HTMLDropup_.appendChild(this.HTMLListUl_);
    this.HTMLDropup_.appendChild(this.HTMLList_);
    this.HTMLButtonsContainer_.appendChild(this.HTMLDropup_);
};

/**
 * remove the content of the table
 */
nsVmap.Map.MapPopup.prototype.removeMessages = function () {
    oVmap.log('nsVmap.Map.MapPopup.prototype.removeMessages');
    this.HTMLContent_.innerHTML = "";
};

/**
 * Remove the popup
 * @param {boolean|undefined} bRemoveFeature false if you dont want to remove the binded feature, default is true
 * @param {boolean|undefined} bRemoveFromPopupsLists false to not auto remove this popup from other popups list
 * @return {boolean}
 */
nsVmap.Map.MapPopup.prototype.remove = function (bRemoveFeature, bRemoveFromPopupsLists) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.remove');

    bRemoveFeature = goog.isDef(bRemoveFeature) ? bRemoveFeature : true;
    bRemoveFromPopupsLists = goog.isDef(bRemoveFromPopupsLists) ? bRemoveFromPopupsLists : true;

    this.HTMLCloser_.blur();

    if (oVmap['properties']['is_mobile']) {
        $('#mobile-popup-bar').empty()
        $('#mobile-popup-bar').addClass('hidden');
    } else {
        this.popupOverlay_.setPosition(undefined);
    }

    // supprime la feature
    if (goog.isDef(this.olFeature_) && bRemoveFeature !== false) {
        if (goog.array.contains(oVmap.getMap().getPopupOverlaySource().getFeatures(), this.olFeature_)){
            oVmap.getMap().getPopupOverlaySource().removeFeature(this.olFeature_);
        }
    }

    // En cas de multipopup
    if (bRemoveFromPopupsLists === true && goog.isArray(this.aPopupsList)) {

        // Retire la popup des listes sur les autres popups
        this.removeFromOtherPopupsList();
    }

    return true;
};

/**
 * Get the center of the geometry
 * @param {ol.geom} geom
 * @returns {Number}
 */
nsVmap.Map.MapPopup.prototype.getGeometryCenter = function (geom) {
    oVmap.log('nsVmap.Map.MapPopup.prototype.getGeometryCenter');

    if (geom.getType() === 'Polygon') {
        var position = geom.getInteriorPoint().getLastCoordinate();
    } else if (geom.getType() === 'MultiPolygon') {

        var aPolygons = geom.getPolygons();
        var maxArea = 0;
        var bigestPolygon = 0;

        // Retrouve le plus gros polygone
        for (var i = 0; i < aPolygons.length; i++) {
            if (maxArea < aPolygons[i].getArea()) {
                maxArea = aPolygons[i].getArea();
                bigestPolygon = i;
            }
        }
        var position = aPolygons[bigestPolygon].getInteriorPoint().getLastCoordinate();

    } else if (geom.getType() === 'LineString') {
        var centerPoint = Math.round(geom.getCoordinates().length / 2);
        var position = geom.getCoordinates()[centerPoint];
    } else if (geom.getType() === 'Circle') {
        var position = geom.getCenter();
    } else if (geom.getType() === 'Point' || geom.getType() === 'MultiPoint') {
        var position = geom.getLastCoordinate();
    } else if (geom.getType() === 'GeometryCollection') {
        return 0;
    } else {
        return geom.getLastCoordinate();
    }

    return position;
};

/**
 * Set popup active / inactive
 * @param  {boolean} bActive
 */
nsVmap.Map.MapPopup.prototype.setActive = function(bActive) {

    var this_ = this;

    this.active = bActive;
    if (this.active === false) {
        this.HTML_.classList.add("unactive");
    } else {
        this.HTML_.classList.remove("unactive");
    }

    if (goog.isDefAndNotNull(this.HTMLPopupsList_)) {
        if (this.active === false) {
            this.HTMLPopupsList_.classList.add("unactive");
        } else {
            this.HTMLPopupsList_.classList.remove("unactive");
        }

        // // Définit dans la liste des popups laquelle est active
        $(this.HTMLPopupsList_).find('.list-group-item').each(function(index){
            this.classList.remove("active");
            if (this.popup === this_) {
                if (this.popup.active === true) {
                    this.classList.add("active");
                }
            }
        })
    }
    if (goog.isDefAndNotNull(this.HTMLPopupsFade_)) {
        if (this.active === false) {
            this.HTMLPopupsFade_.classList.add("unactive");
        } else {
            this.HTMLPopupsFade_.classList.remove("unactive");
        }
    }
}

/**
 * Set la liste des autres popups
 * @param  {array} aPopups
 * @param  {array} aPopupListMenuBtns boutons du menu dépliant
 */
nsVmap.Map.MapPopup.prototype.setPopupsList = function (aPopups, aPopupListMenuBtns) {

    this.aPopupsList = aPopups;

    if (goog.isDefAndNotNull(aPopupListMenuBtns)) {
        this.aPopupListMenuBtns = aPopupListMenuBtns;
    };

    if (goog.isDefAndNotNull(this.aPopupsList)) {
        this.displayPopupsList();
    }
}

/**
 * Handle what to do when user clicks on the popup
 */
nsVmap.Map.MapPopup.prototype.handlePopupClick = function () {

    var this_ = this;

    this.HTML_.onclick = function () {

        // Passe la popup en cours au dessus des autres
        if (!goog.isArray(this_.aPopupsList)) {
            var map = oVmap.getMap().getOLMap();
            var aOverlays = map.getOverlays().getArray();

            // Met tous z-index à 10
            for (var i = 0; i < aOverlays.length; i++) {
                if ($(aOverlays[i].getElement()).hasClass('ol-popup')) {
                    $(aOverlays[i].getElement()).css('z-index', 10);
                }
            }

            // Met le z-index de l'élément clické à 11
            $(this_.HTML_).css('z-index', 11);
        }

        else {
            
            // Rend les autres inactives
            for (var i = 0; i < this_.aPopupsList.length; i++) {
                this_.aPopupsList[i].setActive(false);
            }

            // Rend la popup active
            this_.setActive(true);
        }
    };
}

/**
 * Affiche la liste des autres popups
 */
nsVmap.Map.MapPopup.prototype.displayPopupsList = function () {
    
    if (goog.isArray(this.aPopupsList) && !oVmap['properties']['is_mobile']) {
        
        // Liste
        this.HTMLPopupsList_ = document.createElement('div');
        this.HTMLPopupsList_.classList.add('ol-popups-list');
        if (this.active === false) {
            this.HTMLPopupsList_.classList.add('unactive');
        }
        this.HTML_.parentElement.appendChild(this.HTMLPopupsList_);
        
        // Élément fade (permet de ne pas actionner les boutons lorsque la popup est unactive)
        this.HTMLPopupsFade_ = document.createElement('div');
        this.HTMLPopupsFade_.classList.add('ol-popups-fade');
        if (this.active === false) {
            this.HTMLPopupsFade_.classList.add('unactive');
        }
        this.HTML_.parentElement.appendChild(this.HTMLPopupsFade_);
        
        // Container
        var listContainer = document.createElement('div');
        this.HTMLPopupsList_.appendChild(listContainer);
        
        // Titre
        var listTitle = document.createElement('div');
        listTitle.classList.add('list-title');
        this.translate_('VMAP_BASICSELECT_SELECTIONS_LIST').then(function (sTranslation) {
            listTitle.innerHTML = sTranslation;
        });
        listContainer.appendChild(listTitle);
        
        // Bouton close
        var closeBtn = this.getPopupsListCloseBtn();
        listContainer.appendChild(closeBtn);

        // Menu déroulant
        if (goog.isDefAndNotNull(this.aPopupListMenuBtns)) {
            var actionsMenu = this.getPopupsListMenu(this.aPopupListMenuBtns);
            listContainer.appendChild(actionsMenu);
        }
        
        // Liste
        var listElement = document.createElement('div');
        listElement.classList.add('list-group');
        listContainer.appendChild(listElement);
        
        // Ajoute les éléments dans la liste
        for (var i = 0; i < this.aPopupsList.length; i++) {
            if (goog.isDefAndNotNull(this.aPopupsList[i].tableData) &&
                goog.isDefAndNotNull(this.aPopupsList[i].sMultiPopupField)) {
                if (goog.isDefAndNotNull(this.aPopupsList[i].tableData[this.aPopupsList[i].sMultiPopupField])) {
                    
                    var listItem = this.getPopupsListItem(this.aPopupsList[i]);
                    listElement.appendChild(listItem);
                }
            }
        }
        
        // Set hauteur de l'élément
        $(this.HTMLPopupsList_).height($(this.HTML_).height());
        $(this.HTMLPopupsFade_).height($(this.HTML_).height() + 30);
        $(this.HTMLPopupsFade_).width($(this.HTML_).width() + 160);
        
        // Permet de selectionner la popup quand on clique dessus
        this.HTMLPopupsList_.onclick = this.HTML_.onclick;
        this.HTMLPopupsFade_.onclick = this.HTML_.onclick;
        
    } else {
        console.error('this.aPopupsList not an array or is mobile version');
    }
}

/**
 * Retourne un élément HTML contenant un élément de la liste
 * @param  {object} oPopup
 * @return {object} DOM element
 */
nsVmap.Map.MapPopup.prototype.getPopupsListItem = function (oPopup) {

    // Élément HTML
    var listItem = document.createElement('a');
    listItem.classList.add('list-group-item');
    listItem.classList.add('list-group-item-action');
    if (oPopup === this && this.active === true) {
        listItem.classList.add('active');
    }
    listItem.title = oPopup.tableData[oPopup.sMultiPopupField];
    listItem.innerHTML = oPopup.tableData[oPopup.sMultiPopupField];

    // Mémorisation de la popup dans l'élément HTML
    listItem.popup = oPopup;

    // Événement onclick
    listItem.onclick = angular.bind(this, function(oPopup) {

        var this_ = this;

        // Timeout car il y a déjà un événement sur this.HTMLPopupsList_
        setTimeout(function() {

            // Désactive les autres popups
            for (var i = 0; i < this_.aPopupsList.length; i++) {
                this_.aPopupsList[i].setActive(false);
            }

            // Active la popup
            oPopup.setActive(true);

            // Zoom sur la popup
            oPopup.centerMapOnPopup(true);
        });
    }, oPopup);

    return listItem;
}

/**
 * Retourne le bouton de fermeture de multi-popup
 * @return {object} DOM element
 */
nsVmap.Map.MapPopup.prototype.getPopupsListCloseBtn = function () {

    var this_ = this;

    var closeBtn = document.createElement('a');
    closeBtn.href = '#';
    closeBtn.className = 'ol-popup-closer';

    if (!oVmap['properties']['is_mobile']) {
        closeBtn.className += ' ol-popups-list-closer';
    }

    // Événement click
    closeBtn.onclick = function () {
        if (goog.isArray(this_.aPopupsList)) {
            for (var i = 0; i < this_.aPopupsList.length; i++) {
                this_.aPopupsList[i].remove(true, false);
            }
        }
    };

    return closeBtn;
}

/**
 * Retourne le menu dropup contenant les différents boutons
 * @param  {array} aPopupListMenuBtns boutons du menu dépliant
 */
nsVmap.Map.MapPopup.prototype.getPopupsListMenu = function (aPopupListMenuBtns) {

    // Container
    dropupBtn = document.createElement('div');
    dropupBtn.className = 'dropup ol-popups-list-menu';

    // Liste
    dropupBtnListUl = document.createElement('ul');
    dropupBtnListUl.className = 'dropdown-menu';
    dropupBtn.appendChild(dropupBtnListUl);

    // Bouton dropup
    dropupBtnList = document.createElement('a');
    dropupBtnList.href = '#';
    dropupBtnList.className = 'ol-popups-list-menu-btn icon-format_list_bulleted';
    dropupBtnList.addEventListener('click', angular.bind(this, function (dropupBtnListUl) {
        $(dropupBtnListUl).toggleClass('block');
    }, dropupBtnListUl));
    dropupBtn.appendChild(dropupBtnList);
    
    // Ferme le menu des listes lors du click sur la partie popup simple
    this.HTML_.addEventListener('click', angular.bind(this, function (dropupBtnListUl) {
        $(dropupBtnListUl).removeClass('block');
    }, dropupBtnListUl));

    // Ferme le menu simple lorsqu'un clique sur la partie popup liste
    this.HTMLPopupsList_.addEventListener('click', angular.bind(this, function (dropupBtnListUl) {
        $(this.HTMLListUl_).removeClass('block');
    }, dropupBtnListUl));

    // Ferme le menu lorsqu'on clicke ailleurs que sur la popup
    $('body').click(angular.bind(this, function (dropupBtnList, dropupBtnListUl, e) {
        if ($(dropupBtnList)[0] !== $(e.target)[0]){
            $(dropupBtnListUl).removeClass('block');
        }
    }, dropupBtnList, dropupBtnListUl));

    for (var i = 0; i < aPopupListMenuBtns.length; i++) {
        dropupBtnListLi = document.createElement('li');
        dropupBtnListLiA = document.createElement('a');
        dropupBtnListLiA.href = '#';
        dropupBtnListLiA.innerHTML = aPopupListMenuBtns[i]['content'];
        dropupBtnListLiA.addEventListener('click', aPopupListMenuBtns[i]['event']);
        dropupBtnListLiA.addEventListener('click', angular.bind(this, function (dropupBtnListUl) {
            $(dropupBtnListUl).removeClass('block');
        }, dropupBtnListUl));

        dropupBtnListUl.appendChild(dropupBtnListLi);
        dropupBtnListLi.appendChild(dropupBtnListLiA);
    }

    return dropupBtn;
}

/**
 * Centre la carte sur la popup
 * @param {boolean|undefined} bOnlyIfNotVisible Zoom uniquement si la popup est hors du champ de vision
 */
nsVmap.Map.MapPopup.prototype.centerMapOnPopup = function (bOnlyIfNotVisible) {

    var aExtent = oVmap.getMap().getOLMap().getView().calculateExtent();
    var aCoords = this.popupOverlay_.getPosition();
    var olCoodsGeom = new ol.geom.Point(aCoords);

    if (bOnlyIfNotVisible === true) {
        if (!olCoodsGeom.intersectsExtent(aExtent)) {
            this.map_.getView().setCenter(aCoords);
        }
    } else {
        this.map_.getView().setCenter(aCoords);
    }
}

/**
 * Supprime la popup des listes des autres popups
 */
nsVmap.Map.MapPopup.prototype.removeFromOtherPopupsList = function () {

    var this_ = this;
    
    // En cas de multipopup
    if (goog.isArray(this.aPopupsList)) {
        
        // Index de la popup sur la liste
        var listIndex = this.aPopupsList.indexOf(this);
        
        // Retire la popup en cours de la liste
        this.aPopupsList.splice(listIndex, 1);
        
        // Reset la liste dans chacune des popups
        for (i = 0; i < this.aPopupsList.length; i++) {
            if (oVmap['properties']['is_mobile']) {
                this.aPopupsList[i].setPreNextButtons(this.aPopupsList);
            } else {
                this.aPopupsList[i].setPopupsList(this.aPopupsList);
            }
        }
        
        // Active la prochaine popup
        setTimeout(function() {
            if (goog.isDefAndNotNull(this_.aPopupsList[0])) {
                this_.aPopupsList[0].setActive(true);
                for (i = 1; i < this_.aPopupsList.length; i++) {
                    this_.aPopupsList[i].setActive(false);
                }
            }
        });
    }
}

/**
 * Initialise les boutons précédent et suivant pour la version mobile
 */
nsVmap.Map.MapPopup.prototype.setPreNextButtons = function (aPopups) {

    var this_ = this;
    this.aPopupsList = aPopups;
    if (this.aPopupsList.length > 1) {
        var listIndex = this.aPopupsList.indexOf(this);
    
        var previousIndex = listIndex -1; 
        var nextIndex = listIndex + 1;
    
        var buttons = document.createElement('div');
        buttons.className = 'btns-previous-next';

        var closeBtn = this.getPopupsListCloseBtn();
        this.HTMLButtonsContainer_.remove(this.HTMLCloser_);
        buttons.appendChild(closeBtn);
    
        this.translate_(['VMAP_MAPPOPUP_MOBILE_PREVIOUS', 'VMAP_MAPPOPUP_MOBILE_NEXT']).then(function (aTranslation) {

            if (!(previousIndex < 0)) {
                var previousButton = this_.prepareButton(previousIndex, 'btn-previous', aTranslation['VMAP_MAPPOPUP_MOBILE_PREVIOUS']);
                buttons.append(previousButton);
            }
            if (!(nextIndex > this_.aPopupsList.length - 1)) {
                var nextButton = this_.prepareButton(nextIndex, 'btn-next', aTranslation['VMAP_MAPPOPUP_MOBILE_NEXT']);
                buttons.append(nextButton);
            }

            this_.HTML_.append(buttons);
        });
    }

}

/**
 * Prépare les boutons previous/next pour le mobile
 */
nsVmap.Map.MapPopup.prototype.prepareButton = function (index, additionalClass, additionalText) {
    var this_ = this;
    if (goog.isDefAndNotNull(this.aPopupsList[index])) {
        var btn = document.createElement('button');
        btn.className = 'btn btn-primary ' + additionalClass;
    
        var text = this.aPopupsList[index].tableData[this.aPopupsList[index].sMultiPopupField];
        btn.innerHTML = additionalText + ' (' + (index + 1) + '/' + (this.aPopupsList.length) + ')';
        if (goog.isDefAndNotNull(text)) {
            btn.innerHTML += '<br/>';
            btn.innerHTML += text;
        }
    
        btn.addEventListener('click', function () {
            // Rend les autres inactives
            for (var i = 0; i < this_.aPopupsList.length; i++) {
                this_.aPopupsList[i].setActive(false);
            }
    
            // Rend l'élément précédent actif
            this_.aPopupsList[index].setActive(true);
        });
        return btn;
    }
}

